#!/usr/bin/ruby

PARTITION        = ""       # e.g. /dev/hda or /dev/hda1
RANDOM_POOL      = ""       # e.g. /tmp/random.img
RANDOM_POOL_SIZE = 0.25     # in GB
RANDOM_ROUNDS    = 4        # 
ZERO_POOL        = ""       # e.g. /tmp/zero.img
ZERO_POOL_SIZE   = 0.25     # in GB


# DON'T CHANGE ANYTHING BELOW 


def to_mb( gb )
    return ( gb * 1024 ).to_i
end

def get_size_mb( file )
    size = File.size?( file )
    
    if size.nil?
        return 0
    else
        return size.to_f / ( 1024 ** 2 )
    end
end

# calculate buffer size 2^*MB (max 16MB, max half of pool size)
def calculate_buffer_size
    min_pool_size = [ to_mb( RANDOM_POOL_SIZE ), to_mb( ZERO_POOL_SIZE ) ].min

    if min_pool_size < 5
        return -1
    else
        tmp_bs, limit = 2, min_pool_size - 3         # 3MB as skip range
        while ( ( ( tmp_bs * 2 ) < limit ) and ( ( tmp_bs * 2 ) <= 16 ) )    
            tmp_bs *= 2
        end

        return tmp_bs
    end
end

def write( pool, pool_size )
    round = -1
    count = ( to_mb( pool_size ) - 3 ) / BUFFER_SIZE
    seek  = count - 1

    begin
        round += 1

        command = "dd bs=#{BUFFER_SIZE}M " +
                  "count=#{count} skip=#{rand(4)} " +
                  "seek=#{round * seek} " +
                  "if=#{pool} of=#{PARTITION}"
        puts command

        exec( command + " &> /dev/null" ) if fork.nil?
        Process.wait

    end while ( ( ( round * seek ) + count ) * BUFFER_SIZE < to_mb( PARTITION_SIZE ) and
                $?.exitstatus.zero? )
end

def write_random
    write( RANDOM_POOL, RANDOM_POOL_SIZE )
end

def write_zeros
    write( ZERO_POOL, ZERO_POOL_SIZE )
end

def create_pool( pool, pool_size, source )
    command = "dd bs=1M count=#{to_mb( pool_size )} " +
              "if=#{source} of=#{pool} &> /dev/null"

    exec( command ) if fork.nil?
    Process.wait

    return $?.exitstatus
end

def create_random_pool
    create_pool( RANDOM_POOL, RANDOM_POOL_SIZE, "/dev/urandom" )
end

def create_zero_pool
    create_pool( ZERO_POOL, ZERO_POOL_SIZE, "/dev/zero" )
end



# And action!
PARTITION_SIZE = 10000              # max 10TB partitions
BUFFER_SIZE = calculate_buffer_size

if BUFFER_SIZE < 0
    puts "error: min pool size is 5MB"
    exit 1
end

if get_size_mb( RANDOM_POOL ) == to_mb( RANDOM_POOL_SIZE )
    puts "random pool exists"
else
    puts "creating random pool"
    unless create_random_pool.zero?
        puts "error: couldn't create random pool"
        exit 1
    end
end

if get_size_mb( ZERO_POOL ) == to_mb( ZERO_POOL_SIZE )
    puts "zero pool exists"
else
    puts "creating zero pool"
    unless create_zero_pool.zero?
      puts "error: couldn't create zero pool"
      exit 1
    end
end

RANDOM_ROUNDS.times do |i| 
    puts "writing random (round #{i + 1}/#{RANDOM_ROUNDS})"
    write_random
end

puts "writing zeros"
write_zeros

puts "finnished"
exit 0
